/*
 * libEtPan! -- a mail stuff library
 *
 * Copyright (C) 2001, 2002 - DINH Viet Hoa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the libEtPan! project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef MAIL_STORAGE_H

#define MAIL_STORAGE_H

#include "maildriver_types.h"
#include "mailstorage_types.h"

#include "imapstorage.h"
#include "mboxstorage.h"
#include "mhstorage.h"
#include "nntpstorage.h"
#include "pop3storage.h"

#ifdef __cplusplus
extern "C" {
#endif

/* storage */

/*
  mail_storage_new

  This function creates an empty storage. This storage have to be initialized.
  The "driver" and "data" fields should be initialized.

  @param id  is the name of the storage. It can be NULL.
    The given parameter is no more needed when the creation is finished.
    The given string is duplicated.

  @return The mail storage is returned.
*/

struct mail_storage * mail_storage_new(char * id);

void mail_storage_free(struct mail_storage * storage);

/*
  session will be initialized on success.
*/

int mail_storage_connect(struct mail_storage * storage);

void mail_storage_disconnect(struct mail_storage * storage);


/* folder */

struct mail_folder * mail_folder_new(struct mail_storage * storage,
    char * pathname, char * virtual_name);

void mail_folder_free(struct mail_folder * folder);

int mail_folder_add_child(struct mail_folder * parent,
        struct mail_folder * child);

int mail_folder_detach_parent(struct mail_folder * folder);

int mail_folder_connect(struct mail_folder * folder);

void mail_folder_disconnect(struct mail_folder * folder);


#ifdef __cplusplus
}
#endif

#endif


