/*
 * libEtPan! -- a mail stuff library
 *
 * Copyright (C) 2001, 2002 - DINH Viet Hoa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the libEtPan! project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef POP3DRIVER_TYPES_H

#define POP3DRIVER_TYPES_H

#include <limits.h>

#include "maildriver_types.h"
#include "mailpop3.h"
#include "maildriver_types.h"
#include "chash.h"
#include "mailstorage_types.h"

/* POP3 driver for session */

enum {
  POP3DRIVER_SET_AUTH_TYPE = 1,
};

enum {
  POP3DRIVER_AUTH_TYPE_PLAIN = 0,
  POP3DRIVER_AUTH_TYPE_APOP,
  POP3DRIVER_AUTH_TYPE_TRY_APOP,
};

struct pop3driver_state_data {
  int auth_type;
  mailpop3 * session;
};

/* cached POP3 driver for session */

enum {
  /* the mapping of the parameters should be the same as for pop3 */
  POP3DRIVER_CACHED_SET_AUTH_TYPE = 1,
  /* cache specific */
  POP3DRIVER_CACHED_SET_CACHE_DIRECTORY,
  POP3DRIVER_CACHED_SET_FLAGS_DIRECTORY,
};

struct pop3driver_cached_state_data {
  mailsession * session;
  char cache_directory[PATH_MAX];
  char flags_directory[PATH_MAX];
  chash * flags_hash;
  carray * flags_array;
  struct mail_flags_store * flags_store;
};

/* pop3 storage */

/*
  pop3_mail_storage is the state data specific to the POP3 storage.

  - servername  this is the name of the POP3 server

  - port is the port to connect to, on the server.
      you give 0 to use the default port.

  - connection_type is the type of socket layer to use.
      The value can be CONNECTION_TYPE_PLAIN, CONNECTION_TYPE_STARTTLS,
      CONNECTION_TYPE_TRY_STARTTLS or CONNECTION_TYPE_TLS.
    
  - auth_type is the authenticate mechanism to use.
      The value can be POP3_AUTH_TYPE_PLAIN, POP3_AUTH_TYPE_APOP
      or POP3_AUTH_TYPE_TRY_APOP. Other values are not yet implemented.

  - login is the login of the POP3 account.

  - password is the password of the POP3 account.

  - cached if this value is != 0, a persistant cache will be
      stored on local system.
  
  - cache_directory is the location of the cache.

  - flags_directory is the location of the flags.
*/

struct pop3_mail_storage {
  char * servername;
  uint16_t port;
  char * command;
  int connection_type;

  int auth_type;
  char * login;
  char * password;

  int cached;
  char * cache_directory;
  char * flags_directory;
};

/* this is the type of POP3 authentication */

enum {
  POP3_AUTH_TYPE_PLAIN,             /* plain text authentication */
  POP3_AUTH_TYPE_APOP,              /* APOP authentication */
  POP3_AUTH_TYPE_TRY_APOP,          /* first, try APOP, if it fails,
                                       try plain text */
  POP3_AUTH_TYPE_SASL_ANONYMOUS,    /* SASL anonymous */
  POP3_AUTH_TYPE_SASL_CRAM_MD5,     /* SASL CRAM MD5 */
  POP3_AUTH_TYPE_SASL_KERBEROS_V4,  /* SASL KERBEROS V4 */
  POP3_AUTH_TYPE_SASL_PLAIN,        /* SASL plain */
  POP3_AUTH_TYPE_SASL_SCRAM_MD5,    /* SASL SCRAM MD5 */
  POP3_AUTH_TYPE_SASL_GSSAPI,       /* SASL GSSAPI */
  POP3_AUTH_TYPE_SASL_DIGEST_MD5,   /* SASL digest MD5 */
};


#endif
